import _defineProperty from '@babel/runtime/helpers/defineProperty';
import _JSON$stringify from '@babel/runtime/core-js/json/stringify';
import { createFilter } from 'rollup-pluginutils';
import querystring from 'querystring';
import _regeneratorRuntime from '@babel/runtime/regenerator';
import _asyncToGenerator from '@babel/runtime/helpers/asyncToGenerator';
import _WeakMap from '@babel/runtime/core-js/weak-map';
import _Set from '@babel/runtime/core-js/set';
import _objectSpread from '@babel/runtime/helpers/objectSpread';
import { resolve, dirname, basename, relative } from 'path';
import { parse } from '@vue/component-compiler-utils';
import { createDefaultCompiler, assemble } from '@vue/component-compiler';
import hash from 'hash-sum';

var GET_QUERY = /\.vue(\.[a-z]+?)?\?(.+)$/i;
var PARAM_NAME = 'rollup_plugin_vue';
function createVueFilter(include, exclude) {
  var filter = createFilter(include, exclude);
  return function (id) {
    return filter(id) && id.endsWith('.vue');
  };
}
function getQueryParams(id) {
  var query;
  if (!(query = GET_QUERY.exec(id))) return null;
  query = querystring.parse(query[2]);
  query = query[PARAM_NAME] ? JSON.parse(query[PARAM_NAME]) : null;
  return query;
}
function isVuePartRequest(id) {
  return getQueryParams(id) !== null;
}
function createVuePartRequest(filename, lang, type, index) {
  lang = lang || createVuePartRequest.defaultLang[type];

  var value = _JSON$stringify({
    type: type,
    index: index,
    lang: lang
  });

  var toEncode = _defineProperty({}, PARAM_NAME, value);

  return "".concat(filename, ".").concat(lang, "?").concat(querystring.stringify(toEncode));
}
createVuePartRequest.defaultLang = {
  template: 'html',
  styles: 'css',
  script: 'js'
  /**
   * @export
   * @param {string} id
   * @returns {{filename: string, meta: { type: string, lang: string, index?: number }} | undefined}
   */

};
function parseVuePartRequest(id) {
  if (!id.includes('.vue')) return;
  var length = id.indexOf('.vue') + 4;
  var filename = id.substr(0, length);
  var params = getQueryParams(id);
  if (params === null) return;
  return {
    filename: filename,
    meta: params
  };
}
function resolveVuePart(descriptors, vuePart) {
  var descriptor = descriptors[vuePart.filename];
  var part = descriptor[vuePart.meta.type];
  return Array.isArray(part) ? part[vuePart.meta.index] : part;
}

function vue() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var isVue = createVueFilter(opts.include, opts.exclude);
  var isProduction = process.env.NODE_ENV === 'production';
  createVuePartRequest.defaultLang = _objectSpread({}, createVuePartRequest.defaultLang, opts.defaultLang);
  var shouldExtractCss = opts.css === false;
  var blacklisted = new _Set(opts.blacklistCustomBlocks || ['*']);
  var whitelisted = new _Set(opts.blacklistCustomBlocks || []);

  var isAllowed = function isAllowed(any) {
    return (!blacklisted.has('*') || !blacklisted.has(any)) && (whitelisted.has('*') || whitelisted.has(any));
  };

  delete opts.css;
  delete opts.blacklistCustomBlocks;
  delete opts.defaultLang;
  delete opts.include;
  delete opts.exclude;
  var compiler = createDefaultCompiler(opts);
  var descriptors = new _WeakMap();
  return {
    name: 'vue.delegate',
    resolveId: function resolveId(id) {
      if (isVuePartRequest(id)) {
        var ref = parseVuePartRequest(id);
        var element = resolveVuePart(descriptors, ref);
        if (element.src && ref.meta.type !== 'styles') return resolve(dirname(ref.filename), element.src);
        return id;
      }
    },
    load: function load(id) {
      if (!isVuePartRequest(id)) return;
      id = parseVuePartRequest(id);
      var element = resolveVuePart(descriptors, id);
      return element.code || element.content;
    },
    transform: function () {
      var _transform = _asyncToGenerator(
      /*#__PURE__*/
      _regeneratorRuntime.mark(function _callee(source, filename) {
        var descriptor, scopeId, input, result;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (!isVue(filename)) {
                  _context.next = 10;
                  break;
                }

                descriptor = descriptors[filename] = parse({
                  filename: filename,
                  source: source,
                  needMap: true
                });
                scopeId = 'data-v-' + (isProduction ? hash(basename(filename) + source) : hash(filename + source));
                input = {
                  scopeId: scopeId,
                  styles: descriptor.styles.map(function (style) {
                    return compiler.compileStyle(filename, scopeId, style);
                  }),
                  customBlocks: []
                };

                if (descriptor.template) {
                  input.template = compiler.compileTemplate(filename, descriptor.template);

                  if (input.template.errors && input.template.errors.length) {
                    console.error('> Errors: ' + relative(process.cwd(), filename) + '\n' + input.template.errors.map(function (it) {
                      return '  - ' + it;
                    }).join('\n'));
                  }

                  if (input.template.tips && input.template.tips.length) {
                    console.log('> Tips: ' + relative(process.cwd(), filename) + '\n' + input.template.tips.map(function (it) {
                      return '  - ' + it;
                    }).join('\n'));
                  }
                }

                input.script = descriptor.script ? {
                  code: "\n            export * from '".concat(createVuePartRequest(filename, descriptor.script.lang, 'script'), "'\n            import script from '").concat(createVuePartRequest(filename, descriptor.script.lang, 'script'), "'\n            export default script\n            ")
                } : {
                  code: ''
                };

                if (shouldExtractCss) {
                  input.styles = input.styles.map(function (style, index) {
                    descriptor.styles[index].code = style.code;
                    input.script.code += '\n' + "import '".concat(createVuePartRequest(filename, 'css', 'styles', index), "'");

                    if (style.module || descriptor.styles[index].scoped) {
                      return _objectSpread({}, style, {
                        code: ''
                      });
                    }
                  }).filter(Boolean);
                }

                result = assemble(compiler, filename, input, opts);
                descriptor.customBlocks.forEach(function (block, index) {
                  if (!isAllowed(block.type)) return;
                  result.code += '\n' + "export * from '".concat(createVuePartRequest(filename, block.attrs.lang || createVuePartRequest.defaultLang[block.type] || block.type, 'customBlocks', index), "'");
                });
                return _context.abrupt("return", result);

              case 10:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      return function transform(_x, _x2) {
        return _transform.apply(this, arguments);
      };
    }()
  };
}

export default vue;
//# sourceMappingURL=rollup-plugin-vue.js.map
